# $Id: format-html.tcl,v 1.1 1996/11/18 18:40:03 uwe Exp $
# html format functions

# ------------------------------------------------------------------------------
#
# mime type generation
# texts are prefixed with a mime types for plain text, html, postscript

proc content_type {type doc} {
    return "Content-type: $type\n\n$doc"
}

proc text_plain doc {
    return [content_type "text/plain" $doc]
}

proc text_html doc {
    return [content_type "text/html" $doc]
}

proc text_ps doc {
    return [content_type "application/postscript" $doc]
}

# ------------------------------------------------------------------------------
#
# generates a HTML frame around a page
# with title, header, <HR> and body

proc html_frame {title header body args} {
    setopt bgcolor    ""
    setopt textcolor  ""
    setopt vlinkcolor ""
    setopt linkcolor  ""

    set optlist []
    if {"$bgcolor"!=""    } { lappend optlist "BGCOLOR=$bgcolor"  }
    if {"$textcolor"!=""  } { lappend optlist "TEXT=$textcolor"   }
    if {"$vlinkcolor"!="" } { lappend optlist "VLINK=$vlinkcolor" }
    if {"$linkcolor"!=""  } { lappend optlist "LINK=$vlinkcolor"  }

    set head "\n<HEAD>\n<TITLE>$title ([get_date])</TITLE>\n</HEAD>"
    set body "\n<BODY [join $optlist]>\n$header\n$body\n</BODY>"

    return [tag_html $head$body ]
}

# ------------------------------------------------------------------------------
#
# gen image reference
# format options and defaults are:
#   -align : TOP
#   -alt   : ""
#   -border: 0 pt
#   -hspace: 8 pt
#   -vspace: 0 pt
#   -width : not specified
#   -height: not specified
#   -src   : no extra source file

proc gen_image_ref {dir args} {
    setopt align  TOP
    setopt alt    ""
    setopt border 0
    setopt hspace 8
    setopt src    ""
    setopt vspace 0
    setopt width  ""
    setopt height ""

    set imagename    "$dir"
    if {"$src"!=""}  {append imagename "/$src"}

    set optlist      [ list \
			   "ALIGN=$align" \
			   "ALT=\"$alt\"" \
			   "BORDER=$border" \
			   "HSPACE=$hspace" \
			   "SRC=\"$imagename\"" \
			   "VSPACE=$vspace" \
			  ]
    if {"$width"!="" } { lappend optlist "WIDTH=$width"   }
    if {"$height"!=""} { lappend optlist "HEIGHT=$height" }

    set imageref     ""
    if {[file exists $imagename]} {
	set imageref "\n<IMG [join $optlist]>"
    }

    return $imageref
}

# ------------------------------------------------------------------------------
#
# conversion of a normal string into HTML
# &,<,> and Umlaute

proc special2html s {
    regsub -all "&"  $s "\\\&amp;"   s
    regsub -all "<"  $s "\\\&lt;"    s
    regsub -all ">"  $s "\\\&gt;"    s
    regsub -all "\"" $s "\\\&quot;"  s
    regsub -all \{   $s "\\\&ob;"    s
    regsub -all \}   $s "\\\&cb;"    s
    return $s
}

proc str2html s {
    set s [ special2html $s ]

    regsub -all "" $s "\\\&auml;"  s
    regsub -all "" $s "\\\&Auml;"  s
    regsub -all "" $s "\\\&ouml;"  s
    regsub -all "" $s "\\\&Ouml;"  s
    regsub -all "" $s "\\\&uuml;"  s
    regsub -all "" $s "\\\&Uuml;"  s
    regsub -all "" $s "\\\&szlig;" s

    return $s
}

# ------------------------------------------------------------------------------
#
#transforms all <cr>'s to HTML <BR> and <P> tags

proc cr2html str {
    regsub -all "\\\n\\\n" $str "<P>"    str
    regsub -all "\\\n"     $str "<BR>\n" str
    regsub -all "<P>"      $str "<P>\n"  str
    return $str
}

# ------------------------------------------------------------------------------
#
# generate different fonts and shapes
#
# tag_text: "text" is tagged with "tag" and extra parameters "options"

proc tag_text {text tag args} {
    setopt align ""
    setopt size  ""

    if {"$align"!=""} {
	append tag " ALIGN=$align"
    }
    if {"$size"!=""} {
	append tag " SIZE=$size"
    }

    return "<$tag>$text</[lindex $tag 0]>"
}

proc tag_italic text { return [tag_text $text "I" ] }
proc tag_bold   text { return [tag_text $text "B" ] }
proc tag_tt     text { return [tag_text $text "TT"] }
proc tag_em     text { return [tag_text $text "EM"] }
proc tag_pre    text { return [tag_text $text "PRE"]  }
proc tag_head   text { return [tag_text $text "HEAD"] }
proc tag_html   text { return [tag_text $text "HTML"] }

proc tag_header {text level} { return [tag_text $text "H$level"] }

proc tag_h1     text { return [tag_header $text 1] }
proc tag_h2     text { return [tag_header $text 2] }
proc tag_h3     text { return [tag_header $text 3] }
proc tag_h4     text { return [tag_header $text 4] }
proc tag_h5     text { return [tag_header $text 5] }
proc tag_h6     text { return [tag_header $text 6] }

proc tag_dl     text { return [tag_text $text "DL"] }
proc tag_ul     text { return [tag_text $text "UL"] }

proc tag_a      {text ref}   { return [tag_text $text "A HREF=\"$ref\""] }
proc tag_n      {text ref}   { return [tag_text $text "A NAME=\"$ref\""] }

proc tag_div    {text args}  { return [eval [list tag_text $text "DIV" ] $args] }
proc tag_par    {text args}  { return [eval [list tag_text $text "P"   ] $args] }
proc tag_font   {text args}  { return [eval [list tag_text $text "FONT"] $args] }

# ------------------------------------------------------------------------------
#
# simple tags

proc tag_br   {} {return "\n<BR>"}
proc tag_br_c {} {return "\n<BR CLEAR=ALL>"}
proc tag_p    {} {return "\n<P>\n"}
proc tag_hr   {} {return "\n<HR>"}
proc tag_p_hr {} {return "[tag_p][tag_hr]"}   
proc tag_dt   {} {return "\n<DT>"}
proc tag_dd   {} {return "\n<DD>"}
proc tag_li   {} {return "\n<LI>"}

proc app_latin text {
    if {"$text"!=""} {
	return ", [tag_italic $text]"
    }
    return ""
}

